@@ -10,6 +10,9 @@ use Compiler::Parser::Node::Package;
 use Compiler::Parser::Node::RegPrefix;
 use Compiler::Parser::Node::RegReplace;
 use Compiler::Parser::Node::Regexp;
+use Compiler::Parser::Node::Label;
+use Compiler::Parser::Node::Handle;
+use Compiler::Parser::Node::HandleRead;
 use Compiler::Parser::Node::ForStmt;
 use Compiler::Parser::Node::ForeachStmt;
 use Compiler::Parser::Node::WhileStmt;
@@ -17,7 +20,9 @@ use Compiler::Parser::Node::Function;
 use Compiler::Parser::Node::FunctionCall;
 use Compiler::Parser::Node::IfStmt;
 use Compiler::Parser::Node::ElseStmt;
+use Compiler::Parser::Node::DoStmt;
 use Compiler::Parser::Node::SingleTermOperator;
+use Compiler::Parser::Node::ThreeTermOperator;
 use Compiler::Parser::Node::Array;
 use Compiler::Parser::Node::Hash;
 use Compiler::Parser::Node::Leaf;
@@ -33,7 +38,7 @@ our @ISA = qw(Exporter);
 our %EXPORT_TAGS = ( 'all' => [ qw() ] );
 our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 our @EXPORT = qw();
-our $VERSION = '0.06';
+our $VERSION = '0.07';
 require XSLoader;
 XSLoader::load('Compiler::Parser', $VERSION);
 
